using System;
using Autoscribe.Lyons.GeneralService;
using System.ServiceModel;
using System.ServiceModel.Channels;
using Autoscribe.Lyons.AOAServiceContract;

namespace atmDemo.Wrapper
{
    public class AoaServiceWrapper : IAoaServiceWrapper
    {
        public const string CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME = "AOAServiceSoap";
        public LogonResult Logon(int companyId, string username, string password)
        {
            LogonResult result = null;
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.Logon(companyId, username, password);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<LogonResult>(fe);
                Console.WriteLine(" -->Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new LogonResult(ex.Message);
            }            
            return result;
        }
        public void Logoff(string token)
        {
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    svc.Logoff(token);
                }
            }
            catch (FaultException fe)
            {
                ErrorResult errorData = ParseErrorResult<ErrorResult>(fe);
                Console.WriteLine(" -->Error: {0}", errorData.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
            }
        }

        StatusDescriptionResult IAoaServiceWrapper.DescribeStatusCode(string token, int statusCode)
        {
            return DescribeStatusCode(token, statusCode);
        }

        public DetailedStatusCodeResult CheckAccountOwnershipAndStatus(string token, string returnDetails, string rtn,
            string account, string amount, string customerPresent, string denyNsf, string country, AccountOwner owner)
        {
            DetailedStatusCodeResult result = null;
            var accountStatusRequest = PrepareAccountStatusRequest(rtn, account, amount, customerPresent, denyNsf, country, owner);
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.CheckAccountOwnershipAndStatus(token, returnDetails, accountStatusRequest);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<DetailedStatusCodeResult>(fe);
                Console.WriteLine(" --> Internal Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new DetailedStatusCodeResult(ex.Message);
            }
            return result;
        }

        public DetailedOverallAccountStatusAndOwnershipResults CheckOverallAccountStatusWithOwnersipInfo(string token, string returnDetails, string rtn,
          string account, string amount, string customerPresent, string denyNsf, string country, AccountOwner owner)
        {
            DetailedOverallAccountStatusAndOwnershipResults result = null;
            var accountStatusRequest = PrepareAccountStatusRequest(rtn, account, amount, customerPresent, denyNsf, country, owner);
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.CheckOverallAccountStatusWithOwnersipInfo(token, returnDetails, accountStatusRequest);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<DetailedOverallAccountStatusAndOwnershipResults>(fe);
                Console.WriteLine(" --> Internal Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new DetailedOverallAccountStatusAndOwnershipResults(ex.Message);
            }
            return result;
        }

        public DetailedStatusCodeResult CheckAccountOwnership(string token, string returnDetails, string rtn, string account, string amount, string customerPresent, string denyNsf, string country, AccountOwner owner)
        {
            DetailedStatusCodeResult result = null;
            var accountStatusRequest = PrepareAccountStatusRequest(rtn, account, amount, customerPresent, denyNsf, country, owner);
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.CheckAccountOwnership(token, returnDetails, accountStatusRequest);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<DetailedStatusCodeResult>(fe);
                Console.WriteLine(" --> Internal Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new DetailedStatusCodeResult(ex.Message);
            }
            return result;
        }

        public DetailedStatusCodeResult CheckAccountStatus(string token, string returnDetails, string rtn, string account, string amount, string customerPresent, string denyNsf, string country, AccountOwner owner)
        {
            DetailedStatusCodeResult result = null;
            var accountStatusRequest = PrepareAccountStatusRequest(rtn, account, amount, customerPresent, denyNsf, country, owner);
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.CheckAccountStatus(token, returnDetails, accountStatusRequest);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<DetailedStatusCodeResult>(fe);
                Console.WriteLine(" --> Internal Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new DetailedStatusCodeResult("Internal Error");
            }
            return result;
        }

        public DetailedStatusCodeResultWithDates CheckAccountOwnershipAndStatusWithDates(string token, string returnDetails, string rtn,
           string account, string amount, string customerPresent, string denyNsf, string country, AccountOwner owner)
        {
            DetailedStatusCodeResultWithDates result = null;
            var accountStatusRequest = PrepareAccountStatusRequest(rtn, account, amount, customerPresent, denyNsf, country, owner);
            try
            {

                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.CheckAccountOwnershipAndStatusWithDates(token, returnDetails, accountStatusRequest);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<DetailedStatusCodeResultWithDates>(fe);
                Console.WriteLine(" --> Internal Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new DetailedStatusCodeResultWithDates(ex.Message);
            }
            return result;
        }

        public DetailedStatusCodeResultWithDates CheckAccountOwnershipWithDates(string token, string returnDetails, string rtn, string account, string amount, string customerPresent, string denyNsf, string country, AccountOwner owner)
        {
            DetailedStatusCodeResultWithDates result = null;
            var accountStatusRequest = PrepareAccountStatusRequest(rtn, account, amount, customerPresent, denyNsf, country, owner);
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.CheckAccountOwnershipWithDates(token, returnDetails, accountStatusRequest);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<DetailedStatusCodeResultWithDates>(fe);
                Console.WriteLine(" --> Internal Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new DetailedStatusCodeResultWithDates(ex.Message);
            }
            return result;
        }

        public DetailedStatusCodeResultWithDates CheckAccountStatusWithDates(string token, string returnDetails, string rtn, string account, string amount, string customerPresent, string denyNsf, string country, AccountOwner owner)
        {
            DetailedStatusCodeResultWithDates result = null;
            var accountStatusRequest = PrepareAccountStatusRequest(rtn, account, amount, customerPresent, denyNsf, country, owner);
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.CheckAccountStatusWithDates(token, returnDetails, accountStatusRequest);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<DetailedStatusCodeResultWithDates>(fe);
                Console.WriteLine(" --> Internal Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new DetailedStatusCodeResultWithDates("Internal Error");
            }
            return result;
        }

        private StatusDescriptionResult DescribeStatusCode(string token, int statusCode)
        {
            StatusDescriptionResult result = null;
            try
            {
                using (ChannelFactory<IAOAService> fact = new ChannelFactory<IAOAService>(CFG_ATM_SVC_END_POINT_CONFIGURATION_NAME))
                {
                    IAOAService svc = fact.CreateChannel();
                    result = svc.DescribeStatusCode(token, statusCode);
                }
            }
            catch (FaultException fe)
            {
                result = ParseErrorResult<StatusDescriptionResult>(fe);
                Console.WriteLine(" --> Internal Error: {0}", result.ErrorMesssage);
            }
            catch (Exception ex)
            {
                Console.WriteLine(" --> Internal Error: {0}", ex.Message);
                result = new StatusDescriptionResult(ex.Message);
            }
            return result;
        }

        private AccountStatusRequest PrepareAccountStatusRequest(string rtn, string account, string amount,
            string customerPresent, string denyNsf, string country, AccountOwner owner)
        {
            return new AccountStatusRequest()
            {
                Rtn = rtn,
                AccountNumber = account,
                AccountOwner = owner,
                Amount = amount,
                Country = country,
                CustomerPresent = customerPresent,
                DenyNsf = denyNsf
            };
        }

        private static T ParseErrorResult<T>(FaultException fe) where T : ErrorResult
        {
            T res = (T)typeof(T).GetConstructor(Type.EmptyTypes)?.Invoke(null);
            MessageFault mf = fe.CreateMessageFault();
            ErrorResult errorData;
            if (mf.HasDetail)
            {
                errorData = mf.GetDetail<ErrorResult>();
            }
            else
            {
                errorData = new ErrorResult($"Unknown error: {fe.Code.Name} - {fe.Message}");
            }

            if (res != null)
            {
                res.ErrorMesssage = errorData.ErrorMesssage;
            }
            return res;
        }

    }
}
